************************************************************
*    This is a sample program provided to show some of the
*    capabilities of the M68HC11 Emulator. The program is
*    written as an example of a binary search tree.
*
************************************************************


*************************************************************
*  This is the buffer which is used in this example. The
*  buffer is pre-sorted, which is a prerequisite of a
*  binary search.
*
**************************************************************

        org  $0100
buffer  fill $00,1
        fill $01,1
        fill $03,1
        fill $04,1
        fill $05,1
        fill $08,1
        fill $0a,1
        fill $0c,1
        fill $0d,1
        fill $12,1
        fill $14,1
        fill $1a,1
        fill $1d,1
        fill $21,1
        fill $24,1
        fill $33,1


*******************************************************************
*  This is some working memory space.
*
*******************************************************************

        org  $0200
base        fill $00,2
high         fill $00,2
wrk_sp  fill $00,6



        org  $0500

*******************************************************************
*  This subroutine is used to compute the midpoint between
*  two addresses.
*
*  Incoming:  low address in X
*                    high address in Y
*
*  Outgoing:  midpoint in X
*
*********************************************************************
mid     psha            * save the working registers
        pshb            * save the working registers
                stx  wrk_sp     * save the base address in memory
        xgdy            * exchange the Y and D registers
        stx  wrk_sp+2   * save the base address in memory
        subd  wrk_sp+2  * calculate the delta
        ldx  #$0002     * load X with a two
        idiv                    * divide the delta by two
                xgdx            * exchange D and X registers
        addd wrk_sp     * add D acc. to the base address
        xgdx            * exchange the D acc. and X
                pulb            * restore the working registers
        pula            * restore the working registers
        rts             * return to main
*
*
*  End of the midpoint subroutine.
***************************************************************************



*******************************************************************************
*  This is the beginning of the main program. To see the program work,
*  step through to the ldb #$02 instruction. At this point
*  use the Emulator's functions to change the number to be searched
*  for. Do this by changing the contents of the B accumulator.
*  Set a break point at $0858 and execute. If the number you have
*  entered is found, $aaaa will appear in the X and Y indexing registers.
*  If not found, $0000 will appear. This is the best I could do with little
*  or no I\O capability.
*     A hardware interrupt can be simulated at any time.
*
******************************************************************************
        org  $0800                  * begin the main program
                lds  #$2500                * initialize hardware stack to a safe value
                lda  #$00                    * initialize acc. A to zero
        ldb  #$02                  * load the number to search for
                ldx  #$0100                * load base address of sorted buffer
                stx base                     * save the base address
                ldy  #$010f                 * load high address of sorted buffer
        sty  high                     * save the high address of the buffer
loop1   jsr  mid                       * calculate the midpoint of the search
                stx wrk_sp                 * store the address in the working buffer
                cmpb 0,x                    * are you the number?
                beq found                   * if yes, branch out
                blt  less                      * if lower, branch to proper place
                ldx wrk_sp                 * load the new base address
                inx                              * increment the new base address
                stx  base                   * save the new base address
                ldy high                     * reload the high address
                cmpy base                * check for high and low addresses equal
                blt not_fnd                 * branch out if yes
                bra loop1                   * branch back for another search
less         ldy wrk_sp                 * load the new high address
                dey                            * decrement the new high address
                sty high                     * save the new high address
                ldx base                    * reload the base address
                cmpx high                 * check for high and low addresses equal
        bgt  not_fnd               * branch out if yes
                bra loop1                   * branch back for another search
found       ldx  #$aaaa               * show success
                ldy  #$aaaa               * show success
                bra end                     * jump to the end of the program
not_fnd    ldx  #$0000              * show failure
        ldy  #$0000              * show failure
end           nop                          * end of the main program



***********************************************************************************
*
*   The following interrupt handling routines are simple examples. All they do
*   is fill the registers with identical values. Real handling routines are
*   obviously more complex, yet the ones below are good as examples.
*
***********************************************************************************

        org  $7000                * SWI interrupt subroutine
        lda  #$44                  * This is the software interrupt
        ldb  #$44                  * handler. These routines must
        ldy  #$4444              * be supplied by the programmer.
        ldx  #$4444              * They can be stored at any address.
        rti

        org  $8000                * IRQ interrupt subroutine
        lda  #$55
        ldb  #$55
        ldy  #$5555
        ldx  #$5555
        rti

        org  $9000                * NMI interrupt subroutine
        lda  #$66
        ldb  #$66
        ldy  #$6666
        ldx  #$6666
        rti

**********************************************************************
*   Here are stored the interrupt vectors, both software and
*   hardware. These addresses are inherent to the MC68HC11.
*   The values stored in these addresses are not. The values
*   stored should be the addresses of the corresponding interrupt
*   handling routines.
***********************************************************************

                 org  $ffe6
                 fill $70,1               * SWI interrupt vector  (high byte)
                 fill $00,1               *                                   (low byte)

                 org  $fff2
         fill $80,1               * IRQ interrupt vector (high byte)
         fill $00,1               *                                  (low byte)
         fill $90,1               * NMI interrupt vector (high byte)
         fill $00,1               *                                  (low byte)
