***************************************************************
*    This is a sample program provided to show some of the
*    capabilities of the Motorola Emulator. The program is
*    an example of an hexadecimal bubble sort.
*
***************************************************************


***************************************************************
*
*  Initialize a buffer to be sorted. The number are in random
*  order.
*
****************************************************************
        org  $0100
buffer  fill $0d,1
                fill $04,1
        fill $0e,1
        fill $06,1
        fill $00,1
                fill $0a,1
        fill $08,1
        fill $02,1
        fill $0f,1
        fill $0b,1
        fill $01,1
        fill $07,1
        fill $09,1
        fill $03,1
        fill $05,1
        fill $0c,1


*****************************************************************
*
*  Initialize a memory word to hold the top address of the buffer.
*  This will be used for looping purposes by the bubble sort
*  subroutine.
*
*******************************************************************

                org $0200
top_add   fill $00,2


        org  $0500
************************************************************
*   Store subroutines for later use.
*
************************************************************

************************************************************
*     This subroutine swaps two hexadecimal values.
*
*     Incoming:    value #1 in accumulator b
*                         value #2 pointed at by x
*
*
swap       pshu  a,x           * push the working registers
                lda  0,x              * get value #2
                stb  0,x              * save value #1
        sta  0,-x             * save value #2
                pulu a,x             * restore the working registers
done        rts
*
*        End the swap subroutine
*************************************************************

************************************************************
*     This subroutine sorts a set of hexadecimal values
*     using the bubble sort algorithm.
*
*     Incoming:    cardinality of the set  in accumulator B
*                         base address of the set in X-Indexing register
*
*
*
bubble     pshu  a,b,y        * push the working registers
                cmpb #$02        * test for at least two items to sort
                blt exit               * stop, if not at least two items to sort
                tfr x,y                 * save the base address
                decb                  * decrement the B accumulator
                abx                    * calculate the top address
                stx top_add       * store the top address
loop1       lda  #$00           * prepare the loop1 stopping variable
                tfr y,x                 * reload the base address
loop2       ldb  0,x+            * get a value from memory
                cmpb 0,x           * compare two adjacent values
                ble test              * branch down if no swap necessary
                lbsr swap          * swap the two adjacent values
                lda  #$01           * load the swap flag
test          cmpx top_add   * check for a complete pass
                bne  loop2         * branch back if pass not complete
                cmpa  #$00       * check the swap flag
                bne  loop1         * branch back if a swap occurred
                pulu a,b,dp,y     * restore the working register
exit          rts                     * return to main
*
*
*         End the bubble sort subroutine
*************************************************************



******************************************************************
*
*  Beginning of the main program. All of the registers are not
*  used in this program. However, all are initialized to show
*  the working of the Emulator. The stacks are initialized to
*  values which do not conflict with other subroutines or
*  interrupt handlers. A hardware interrupt can be simulated
*  at any time during program stepping. To see the program
*  run to completion, set a break at $1016 (nop opcode) and
*  execute.
*
*

        org  $1000               * begin the main program
                lda  #$ff               * initialize acc. A to an arbitrary value
                ldu  #$2000             * initialize U to a safe stack value
                lds  #$2500             * initialize S to a safe stack value
                ldb  #$10                 * initialize acc. B to the cardinality of the set
        ldy  #$0000             * initialize Y to zero
                leax buffer,y            * initialize X to the base buffer address
                lbsr  bubble             * call the bubble sort routine
                nop                          * end the main program


        org  $4000               * NMI interrupt subroutine
        lda  #$11                 * This is a simple interrupt handling
        ldb  #$11                 * routine to load all registers
        ldx  #$1111             * with ones. The programmer must
        ldy  #$1111             * supply all interrupt handlers
        ldu  #$1111
        rti

        org  $5000                * IRQ interrupt subroutine
        lda  #$22                  * This is a simple interrupt handling
        ldb  #$22                  * routine to load all registers
        ldx  #$2222              * with twos. This routine must
        ldu  #$2222      * be supplied by the
        ldy  #$2222      * programmer
        rti

        org  $6000                * FIRQ interrupt subroutine
        lda  #$33                  * This is another simple interrupt
        ldb  #$33                * handling routine.
        ldy  #$3333
        ldu  #$3333
        ldx  #$3333
        rti

        org  $7000                * SWI interrupt subroutine
        lda  #$44                  * This is the software interrupt
        ldb  #$44                  * handler. These routines must
        ldy  #$4444              * be supplied by the programmer.
        ldx  #$4444              * They can be stored at any address.
        ldu  #$4444
        rti

        org  $8000                * SWI2 interrupt subroutine
        lda  #$55
        ldb  #$55
        ldy  #$5555
        ldu  #$5555
        ldx  #$5555
        rti

        org  $9000                * SWI3 interrupt subroutine
        lda  #$66
        ldb  #$66
        ldy  #$6666
        ldx  #$6666
        ldu  #$6666
        rti

***************************************************************
*  Here are stored the interrupt vectors, both software and
*  hardware. These addresses are inherent to the M6809. The
*  values stored in these addresses are not. The values stored
*  should be the addresses of the corresponding interrupt
*  handling routines.
*
****************************************************************

         org  $fff2
         fill $90,1               * SWI3 interrupt vector (high byte)
         fill $00,1               *                                    (low byte)
         fill $80,1               * SWI2 interrupt vector (high byte)
         fill $00,1               *                                    (low byte)
         fill $60,1               * FIRQ interrupt vector (high byte)
         fill $00,1               *                                    (low byte)
         fill $50,1               * IRQ interrupt vector   (high byte)
         fill $00,1               *                                    (low byte)
         fill $70,1               * SWI interrupt vector   (high byte)
         fill $00,1               *                                    (low byte)
         fill $40,1               * NMI interrupt vector   (high byte)
