************************************************************             
*    This is a sample problem provided to show some of the
*    capabilities of the M68HC11 Emulator. The program is
*    written only as an example.
*
************************************************************

	org  $00a0           * start here to show direct  
	fill $00,1              * page addressing mode
	fill $a8,1              * initialize a block to memory
	fill $33,1              * to be used in indexed addressing
	fill $03,1              * among other things
	fill $72,1
	fill $03,1
	fill $74,1
	fill $03,1
	fill $76,1
	fill $03,1
	fill $78,1
	fill $03,1
	fill $7a,1
	fill $03,1
	fill $7c,1

	org  $00c0
		fcc 'This is an example'
		fcc ' program provided for'
		fcc ' the Motorola M68HC11 Emulator'

	org $0100
	bsz  $10          * initialize a block of zeroes 

	org  $0270       * initialize another block for
	fill $03,1          * general usage
	fill $86,1
	fill $22,1
	fill $33,1
	fill $44,1
	fill $55,1
label1      fill $66,1          * a label example
	fill $77,1
	fill $88,1
	fill $99,1
label2      fill $aa,1          * another one
	fill $bb,1
	fill $cc,1
	fill $dd,1

	org  $0370          * initialize a third block of storage
	fill $fa,1
	fill $33,1
	fill $8a,1
	fill $34,1
	fill $45,1
	fill $c7,1
	fill $67,1
label3      fill $78,1
	fill $00,1
	fill $9a,1
	fill $ab,1
	fill $e0,1
	fill $cd,1
	fill $de,1

	org  $0500
************************************************************
*   Store three subroutines for later use.
*
************************************************************

************************************************************
*     This subroutine takes in an integer in the accumulator
*     b and converts it to its absolute value. The new value
*     is returned in the accumulator b.
*
abs          orb  #$00
	bpl  done
	eorb #$ff
	lda #$f7
	tap
	adcb  #1
done        rts
*     End the absolute value routine
*************************************************************

***********************************************************
*    This subroutine stores whatever value is in
*    the accumulator A in five address locations
*    beginning at the location passed in the x register

store        ldb  #$5
again       sta  0,x+
	decb
	bne  again
	rts
*    End the store subroutine
************************************************************

**********************************************************
*    This subroutine reads the values in the five locations
*    beginning at the address in the y register and complements
*    each.

rev           ldb  #$5
again2     lda  0,y
	coma
	sta  0,y+      
	decb
	bne  again2
	rts
*     End the reverse subroutine
************************************************************

	org  $0800                  * begin the main program
	lds  #$1000                * initialize the hardware stack
	lda  #$00                    * load A accumulator (immediate addr.)
	tap                              * enable interrupts
	ldaa  #$04                  * load A acc. (immediate addr.)
	ldy  $0373                  * load Y indx. (extended addr.)
	ldab  $a2                    * load B acc. (direct page addr.)
	ldx  #$0270                * load X indx. (immediate addr.)
	lds  4,x                       * load S ptr. (indexed addr.)
	suba #$04                  * subtract from A (immediate addr.)
	sec                             * set the carry flag    
	addb $0371                * add to B (extended addr.)  
	ldy  #$c000                * load Y indx. (immediate addr.)
	sta  0,y                      * save A acc. (indexed addr.)
	stb  1,y                      * save B acc. (indexed addr.)
	ldd  #$0000               * load D acc. (immediate addr.)
	subd #$0001             * subtract from D (immediate addr.) 
	std  $a2                     * save D in memory (direct page addr.)
	ldd  $0370                 * load D acc. (extended addr.)
	ldd  $a2                     * reload D acc. (direct page addr.)
	bra brch1                   * branch always to brch1
                lda  #$04                   * load A acc. (immediate addr.)
brch4       deca                          * decrement the A acc.
	beq brch3                  * branch, if Z flag is set
	bra brch4                   * branch always to brch4 
	ldab  #$80                  * load B acc. (immediate addr.)
	sei                             * set the I flag
	lds  #$1000               * load the hardware stack (immediate addr.)
	pshx                          * push the X-indexing register
	pshy                          * push the Y-indexing register
	ldx  #$aaaa               * load X-indexing register
	ldy  #$bbbb               * load Y-indexing register
	puly                           * pull the Y-indexing register
	pulx                           * pull the X-indexing register  
	ldaa  #$03                 * load A acc. (immediate addr.)
                ldab  #$05                 * load B acc. (immediate addr.)
	rora                           * rotate right the A accumulator
	rola                           * rotate left the A accumulator   
brch6       rorb                           * rotate the one to the right             
	decb                          * decrement the B acc.
	tba                             * transfer acc. B to acc. A
	beq brch5                  * branch, if Z flag is set
	bra brch6                  * branch always to brch6
brch3       orab $00a0                * or the B acc. (extended addr.)
	swi                            * demo software interrupt
	ldy  #$0370              * load Y indx. (immediate addr.)
	anda 2,y                   * and A acc. (indexed addr.)
                bra  brch1                 * branch to the bottom
brch5       ldb $a3                     * load B acc. (direct page addr.)
	jsr abs                     * jump to the abs subroutine
	clra                           * clear the A acc.
	clrb                           * clear the B acc.
	jsr rev                      * jump to the rev subroutine              
	tsy                            * transfer the stack pointer to Y
	txs                            * transfer X to the stack pointer 
	ldd  $0372                * load D accumulator (extended addr.)
	xgdy                         * exchange D acc. and Y
	tst $037a                  * test the memory location
	sts $0080                 * store the S ptr. in a new location
	eorb $00a5               * exc. or the B acc. (dir. page addr.)        
	ldx  #$bb00              * load the X indx. (immediate addr.)
	jmp brch3                  * jump back 
brch1       ldx  label2                * load X inx. (extended addr.)
                inx                            * increment X indexing register
                tsx                            * transfer S pointer to X register
                ldaa  #$04                * load A acc. (immediate addr.)
                asla                          * arithmetic shift acc. A
                coma                        * complement acc. A
                eora  $0372              * exclusive or acc. A
                staa  $0372              * store acc. A (extended addr.)
                ldaa  $0274              * load the A acc. (extended addr.)
	jsr store                   * jump to the store subroutine
	tpa                            * transfer condition code to acc A.
                ldy  #$0276              * load Y indexing register 
                cmpa 0,y                  * compare acc. A to Y indexing(indexed addr.)
                ldab  #$80                * load acc. B(immediate addr.)
brch9       clc                            * clear the carry flag 
                rorb                          * rotate right acc. B
                beq   brch9              * branch back if zero
                nop                          * end of program 


***********************************************************************************
*
*   The following interrupt handling routines are simple examples. All they do
*   is fill the registers with identical values. Real handling routines are 
*   obviously more complex, yet the ones below are good as examples.
*
***********************************************************************************

	org  $7000                * SWI interrupt subroutine
	lda  #$44                  * This is the software interrupt
	ldb  #$44                  * handler. These routines must
	ldy  #$4444              * be supplied by the programmer.
	ldx  #$4444              * They can be stored at any address.
	rti

	org  $8000                * IRQ interrupt subroutine
	lda  #$55  
	ldb  #$55
	ldy  #$5555
	ldx  #$5555
	rti

	org  $9000                * NMI interrupt subroutine
	lda  #$66
	ldb  #$66
	ldy  #$6666
	ldx  #$6666
	rti

**********************************************************************               
*   Here are stored the interrupt vectors, both software and 
*   hardware. These addresses are inherent to the MC68HC11. 
*   The values stored in these addresses are not. The values 
*   stored should be the addresses of the corresponding interrupt
*   handling routines.
***********************************************************************

                 org  $ffe6 
                 fill $70,1               * SWI interrupt vector  (high byte)
                 fill $00,1               *                                   (low byte)

                 org  $fff2
	 fill $80,1               * IRQ interrupt vector (high byte)
	 fill $00,1               *                                  (low byte)
	 fill $90,1               * NMI interrupt vector (high byte)
	 fill $00,1               *                                  (low byte)
	