************************************************************             
*    This is a sample problem provided to show some of the
*    capabilities of the M6809 Emulator. The program is
*    written only as an example.
*
************************************************************

	org  $00a0           * start here to show direct  
	fill $00,1              * page addressing mode
	fill $a8,1              * initialize a block to memory
	fill $33,1              * to be used in indexed addressing
	fill $03,1              * among other things
	fill $72,1
	fill $03,1
	fill $74,1
	fill $03,1
	fill $76,1
	fill $03,1
	fill $78,1
	fill $03,1
	fill $7a,1
	fill $03,1
	fill $7c,1

	org  $00b0
	fcc 'This is an example program'
                fcc ' provided with the Motorola'
                fcc ' M6809 Emulator.'

                org  $0100
                bsz  $10          * initialize a block of zeroes

                org  $0200
	fill  $dd, 25      * initialize a block of 25 $dd       

	org  $0270       * initialize another block for
	fill $03,1          * general usage
	fill $86,1
	fill $22,1
	fill $33,1
	fill $44,1
	fill $55,1
label1      fill $66,1          * a label example
	fill $77,1
	fill $88,1
	fill $99,1
label2      fill $aa,1          * another one
	fill $bb,1
	fill $cc,1
	fill $dd,1

	org  $0370          * initialize a third block of storage
	fill $fa,1
	fill $33,1
	fill $8a,1
	fill $34,1
	fill $45,1
	fill $c7,1
	fill $67,1
label3      fill $78,1
	fill $00,1
	fill $9a,1
	fill $ab,1
	fill $e0,1
	fill $cd,1
	fill $de,1

	org  $0500
************************************************************
*   Store three subroutines for later use.
*
************************************************************

************************************************************
*     This subroutine takes in an integer in the accumulator
*     b and converts it to its absolute value. The new value
*     is returned in the accumulator b.
*
abs          orb  #$00
	bpl  done
	eorb #$ff
	andcc #$f7
	adcb  #1
done        rts
*     End the absolute value routine
*************************************************************
***********************************************************
*    This subroutine stores whatever value is in
*    the accumulator A in five address locations
*    beginning at the location passed in the x register

store        ldb  #$5
again       sta  0,x+
	decb
	bne  again
	rts
*    End the store subroutine
************************************************************
**********************************************************
*    This subroutine reads the values in the five locations
*    beginning at the address in the y register and complements
*    each.

rev           ldb  #$5
again2     lda  0,y
	coma
	sta  0,y+      
	decb
	bne  again2
	rts
*     End the reverse subroutine
************************************************************

	org  $0800                  * begin the main program
	lda  #$04                    * load A acc. (immediate addr.)
	ldy  $0373                  * load Y indx. (extended addr.)
	ldb  $a2                      * load B acc. (direct page addr.)
	ldu  $0372                  * load U ptr. (extended addr.)
	ldx  #$0270                * load X indx. (immediate addr.)
	lds  a,x                       * load S ptr. (indexed addr.)
	pshu a,b,y,s               * push four registers 
	suba #$04                  * subtract from A (immediate addr.)
	andcc #$00                 * clear all flags
	addb $0371                * add to B (extended addr.)  
	ldy  #$c000                * load Y indx. (immediate addr.)
	sta  0,y+                    * save A acc. (postincrement addr.)
	stb  0,y+                    * save B acc. (postincrement addr.)
	pulu a,b,y,s               * pull the registers
	ldd  #$0000               * load D acc. (immediate addr.)
	subd #$0001             * subtract from D (immediate addr.) 
	std  $a2                     * save D in memory (direct page addr.)
	ldd  $0370                 * load D acc. (extended addr.)
	ldd  $a2                     * reload D acc. (direct page addr.)
	lbra brch1                  * branch always to brch1
brch2       lda  #$04                   * load A acc. (immediate addr.)
brch4       deca                          * decrement the A acc.
	beq brch3                  * branch, if Z flag is set
	bra brch4                   * branch always to brch4 
	ldb  #$80                   * load B acc. (immediate addr.)
	andcc #$fe                * clear the carry flag
	lda  #$03                   * load A acc. (immediate addr.) 
brch6       rorb                           * rotate the one to the right             
	deca                          * decrement the A acc.
	beq brch5                  * branch, if Z flag is set
	bra brch6                  * branch always to brch6
brch3       orb $00a0                 * or the B acc. (extended addr.)
                swi3                          * demo software interrupt, swi3
	ldy  #$0370              * load Y indx. (immediate addr.)
	anda 2,y                   * and A acc. (indexed addr.)
brch5       ldb $a3                     * load B acc. (direct page addr.)
	lbsr abs                     * branch to the abs subroutine
	clra                           * clear the A acc.
	clrb                           * clear the B acc.
	lbsr rev                      * branch to the rev subroutine              
	exg x,y                     * swap the X and Y indx. registers
                swi2                          * demo software interrupt, swi2
	tst $037a                  * test the memory location
	sts $0080                 * store the S ptr. in a new location
	eorb $00a5               * exc. or the B acc. (dir. page addr.)        
                ldx  #$bb00              * load the X indx. (immediate addr.)
                jmp exit                    * jump the lbra instruction 
brch1       ldx  label2                * load X inx. (extended addr.)
                lbra brch2                 * branch always to brch2
exit          lda  $0274                * load the A acc. (extended addr.)
	lbsr store                  * branch to the store subroutine
	tfr  a,b                      * transfer the A acc. to the B acc.
	clra		 * clear the A accumulator
	clrb		 * clear the B accumulator
	lda  #$81                  * load the A accumulator (immediate addr.)
	rola		 * rotate left the A accumulator
	rola		 * rotate left the A accumulator

	org  $4000               * NMI interrupt subroutine
	lda  #$11                 * this is a simple interrupt handling
	ldb  #$11                 * routine to load all registers
	ldx  #$1111              * with ones. The programmer must
	ldy  #$1111              * supply all interrupt handlers
	ldu  #$1111
	rti

	org  $5000                * IRQ interrupt subroutine
	lda  #$22
	ldb  #$22
	ldx  #$2222
	ldu  #$2222
	ldy  #$2222
	rti
	       
	org  $6000                * FIRQ interrupt subroutine
	lda  #$33  
	ldb  #$33
	ldy  #$3333
	ldu  #$3333
	ldx  #$3333
	rti

	org  $7000                * SWI interrupt subroutine
	lda  #$44                  * This is the software interrupt
	ldb  #$44                  * handler. These routines must
	ldy  #$4444               * be supplied by the programmer.
	ldx  #$4444               * They can be stored at any address.
	ldu  #$4444
	rti

	org  $8000                * SWI2 interrupt subroutine
	lda  #$55  
	ldb  #$55
	ldy  #$5555
	ldu  #$5555
	ldx  #$5555
	rti

	org  $9000                * SWI3 interrupt subroutine
	lda  #$66
	ldb  #$66
	ldy  #$6666
	ldx  #$6666
	ldu  #$6666
	rti

***************************************************************               
*  Here are stored the interrupt vectors, both software and 
*  hardware. These addresses are inherent to the M6809. The
*  values stored in these addresses are not. The values stored
*  should be the addresses of the corresponding interrupt
*  handling routines.
*  
****************************************************************

	 org  $fff2
	 fill $90,1               * SWI3 interrupt vector (high byte)
	 fill $00,1               *                       (low byte)
	 fill $80,1               * SWI2 interrupt vector (high byte)
	 fill $00,1               *                       (low byte)
	 fill $60,1               * FIRQ interrupt vector (high byte)
	 fill $00,1               *                       (low byte)
	 fill $50,1               * IRQ interrupt vector  (high byte)
	 fill $00,1               *                       (low byte)
	 fill $70,1               * SWI interrupt vector  (high byte)
	 fill $00,1               *                       (low byte)
	 fill $40,1               * NMI interrupt vector  (high byte)
	 fill $00,1                                       (low byte)
